package org.example;

import org.graphstream.algorithm.Toolkit;
import org.graphstream.algorithm.generator.Generator;
import org.graphstream.algorithm.generator.RandomGenerator;
import org.graphstream.graph.Graph;
import org.graphstream.graph.implementations.DefaultGraph;
import org.graphstream.graph.implementations.SingleGraph;
import org.graphstream.stream.file.FileSource;
import org.graphstream.stream.file.FileSourceEdge;

import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Arrays;
import java.util.OptionalDouble;

import static org.graphstream.algorithm.Toolkit.averageDegree;

public class propagation {
    public static double seuilEpidemiqueReseauAleatoire(double degreMoyen, int nbNoeud){
        System.out.println("Réseau aleatoire :");
        Graph graphe = new SingleGraph("Random");
        Generator gen = new RandomGenerator(degreMoyen);
        gen.addSink(graphe);
        gen.begin();
        for (int i = 0; i < nbNoeud; i++)
            gen.nextEvents();
        gen.end();
        double seuilEpidemique = 1 / (degreMoyen + 1);
        System.out.println("le seuil epidemique est : " + seuilEpidemique);
        return seuilEpidemique;
    }
    public static double seuilEpidemique(Graph graph){
        System.out.println("Notre réseau :");
        double degreMoyen = (float) averageDegree(graph);
        System.out.println("le degre moyen du graphe est : "+ degreMoyen);
        int[] dd = Toolkit.degreeDistribution(graph);
        OptionalDouble distributionDegre = Arrays.stream(dd).average();
        System.out.println("Distribution de degré : " + distributionDegre);
        double distribution = distributionDegre.getAsDouble();
        double seuilEpidemique = degreMoyen / distribution;
        System.out.println("le seuil epidemique est : " + seuilEpidemique);
        return seuilEpidemique;

    }
    public static void main(String[] args){
        String filePath = "com-dblp.ungraph.txt";
        Graph g = new DefaultGraph("g");
        FileSource fs = new FileSourceEdge();
        fs.addSink(g);
        System.setProperty("org.graphstream.ui", "swing");
        try {
            fs.readAll(filePath);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            fs.removeSink(g);
        }
        int nbNoeuds = g.getNodeCount();
        System.out.println("le nombre de noeuds du graphe est : "+ nbNoeuds);
        int nbEdges = g.getEdgeCount();
        System.out.println("le nombre de liens du graphe est : "+ nbEdges);
        double degreMoyen = averageDegree(g);
        System.out.println("le degre moyen du graphe est : "+ degreMoyen);
        double B = 0.1428;
        System.out.println("la probabilité de trassmission : "+ B);
        double U = 0.0714;
        System.out.println("la probabilité d'etre remis : "+ U);
        double landa = B/U;
        System.out.println("le taux de propagation du virus : "+ landa);
        double seuil = seuilEpidemique(g);
        double seuilAleatoire = seuilEpidemiqueReseauAleatoire(degreMoyen, nbNoeuds);
        if(seuil > seuilAleatoire){
            System.out.println("le seuil epidemique de notre reseau est plus grand que le seuil epidemique du reseau aleatoire");
        }else if(seuil < seuilAleatoire){
            System.out.println("le seuil epidemique de notre reseau est plus petit que le seuil epidemique du reseau aleatoire");
        }else{
            System.out.println("le seuil epidemique de notre reseau et le seuil epidemique du reseau aleatoire sont equivalents");
        }

    }
}
